import 'dart:async';
import 'dart:io';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:kova_resources/src/repository.dart';
import 'package:kova_resources/src/utils.dart';
import 'package:firebase_storage/firebase_storage.dart';

class DBFireStore implements RepositorySource, RepositoryCache {
  Firestore store;
  Utils utils;

  DBFireStore() {
    init();
  }

  Future<void> init() async {
    store = Firestore.instance;
    utils = Utils();
  }

  Future<List<Map<String, dynamic>>> fetchDocuments(String collection) async {
    List<Map<String, dynamic>> list = [];
    return store.collection(collection).getDocuments().then((QuerySnapshot snapshot) {
      for (DocumentSnapshot document in snapshot.documents) {
        Map<String, dynamic> map = document.data;
        list.add(map);
      }
      return list;
    });
  }

  Future<List<Map<String, dynamic>>> fetchDocumentsWhere(String collection, String field, String equals) async {
    List<Map<String, dynamic>> list = [];
    return store.collection(collection).where(field, isEqualTo: equals).getDocuments().then((QuerySnapshot snapshot) {
      for (DocumentSnapshot document in snapshot.documents) {
        Map<String, dynamic> map = document.data;
        list.add(map);
      }
      return list;
    });
  }

  Future<Map<String, dynamic>> fetchDocument(String collection, String uid) async {
    return store.collection(collection).document(uid).get().then((DocumentSnapshot snapshot) {
      return snapshot.data;
    });
  }

  Future<int> addItem(String collection, String uid, Map<String, dynamic> item) async {
    store.collection(collection).document(uid).setData(item, merge: true).then((_) {
      return 0;
    });
    return 0;
  }

  Future<int> clear(String collection) async {
    return 0;
  }

  Future<String> storeFile(String collection, String inspectionId, File file, String contentType) async {
    String fileName = utils.getFileName(file);
    final StorageReference storageRef = FirebaseStorage.instance.ref().child('$collection/$inspectionId/$fileName');
    final StorageUploadTask uploadTask = storageRef.putFile(file, StorageMetadata(contentType: contentType));
    return uploadTask.onComplete.then((StorageTaskSnapshot snapshot) {
      return snapshot.ref.getDownloadURL().then((dynamic url) {
        return url.toString();
      }).catchError((onError) {
        return '';
      });
    });
  }

  Future<void> deleteFile(String fileUrl, String fileThumbUrl) async {
    final StorageReference fullStorageRef = FirebaseStorage.instance.ref().child(fileUrl);
    final StorageReference thumbStorageRef = FirebaseStorage.instance.ref().child(fileThumbUrl);

    return fullStorageRef.delete().then((_) {
      return thumbStorageRef.delete().then((_) {
        return;
      }).catchError((onError) {
        return;
      }).whenComplete(() {});
    }).catchError((onError) {
      return;
    }).whenComplete(() {});
  }

  Future<void> updateDocument(String collection, String uid) async {
    collection = 'users';
    uid = 'fZvOSDHHz4ShcILkjftY2Mo7mNW2';
    uid = 'hiNZhLZdkvTuChf8qoSn';

    //print("\x1B[2J\x1B[0;0H");

    String documentPath = '$collection/$uid';
    DocumentReference documentReference = store.document(documentPath);

    if (documentReference == null) {
      print('document is null  -- use set');
    } else {
      print('document exists  -- use update');
    }

    var batch = store.batch();

    Map<String, dynamic> map = {'status': 'Rejected'};

    batch.setData(documentReference, map);
    //Rbatch.updateData(documentReference, map);
    //batch.updateData(documentReference, map);
    batch.commit();

    // String documentPath = '$collection/$uid';

    // final DocumentReference documentReference = store.document(documentPath);

    // store.runTransaction((Transaction transanction) async {
    //transaction.get(documentReference).
    // });

    // Firestore.instance.runTransaction((Transaction tran) {
    //   tran.get(postRef).then((DocumentSnapshot snap) {
    //     if (snap.exists) tran.update(postRef, <String, dynamic>{"likesCount": snap.data['likesCount'] + 1});
    //   });
    // });

    // final DocumentReference postRef = Firestore.instance.document('posts/123');
    // Firestore.instance.runTransaction((Transaction tx) async {
    //   DocumentSnapshot postSnapshot = await tx.get(postRef);
    //   if (postSnapshot.exists) {
    //     await tx.update(postRef, <String, dynamic>{'likesCount': postSnapshot.data['likesCount'] + 1});
    //   }
    // });
  }
}
