import 'dart:io';
import 'dart:async';
import 'package:kova_resources/src/preferences.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';
import 'package:path_provider/path_provider.dart';
import 'package:kova_resources/src/repository.dart';
import 'package:sqflite_migration/sqflite_migration.dart';

class DBSQLite implements RepositorySource, RepositoryCache {
  Database db;
  List<String> initialScript;
  List<String> migrationScripts;
  bool dbIsReady = false;
  Preferences preferences;

  DBSQLite(Preferences preferences) {
    init(initialScript, migrationScripts);
  }

  void init(List<String> initialScript, List<String> migrationScripts) async {
    Directory documentsDirectory = await getApplicationDocumentsDirectory();
    final path = join(documentsDirectory.path, 'sql.db');
    final config = MigrationConfig(initializationScript: initialScript, migrationScripts: migrationScripts);
    openDatabaseWithMigration(path, config).then((Database database) {
      db = database;
      this.dbIsReady = true;
    });
  }

  Future<List<Map<String, dynamic>>> fetchDocuments(String collection) async {
    return null;
  }

  Future<Map<String, dynamic>> fetchDocument(String collection, String uid) async {
    if (this.dbIsReady) {
      final maps = await db.query(collection, columns: null, where: "firebaseId = ?", whereArgs: [uid]);
      if (maps.length > 0) {
        return maps.first;
      }
    }
    return null;
  }

  Future<List<Map<String, dynamic>>> fetchInspectionsWhere(String collection, String field, String equals, bool pendingOnly) async {
    List<Map<String, dynamic>> list = [];
    if (this.dbIsReady) {
      final maps = await db.query(collection, columns: null, where: "$field = ?", whereArgs: [equals]);
      for (Map<String, dynamic> map in maps) {
        if (pendingOnly) {
          if (map['completed'] == null || map['completed'] == 0) {
            list.add(map);
          }
        } else {
          if (map['completed'] == 1) {
            list.add(map);
          }
        }
      }
      return list;
    }
    return null;
  }

  Future<int> addItem(String collection, String uid, Map<String, dynamic> item) async {
    if (this.dbIsReady) {
      return db.insert(collection, item, conflictAlgorithm: ConflictAlgorithm.replace);
    }
    return 0;
  }

  Future<int> clear(String collection) async {
    return db.delete(collection);
  }

  Future<String> storeFile(String collection, String inspectionId, File file, String contentType) async {
    // String fileName = App.utils.getFileName(file);
    // Directory storageDirectory = await App.utils.getStorageDirectory();
    // await file.copy('${storageDirectory.path}/storage/$fileName');
    return '';
  }
}
