import 'dart:async';
import 'dart:io';
//import 'package:kova_resources/src/db_sqlite.dart';
import 'package:kova_resources/src/db_firestore.dart';

class Repository {
  Repository() {
    init();
  }

  init() async {}

  List<RepositorySource> sources = <RepositorySource>[
    //DBSQLite(),
    DBFireStore(),
  ];

  List<RepositoryCache> caches = <RepositoryCache>[
    //DBSQLite(),
    DBFireStore(),
  ];

  Future<List<Map<String, dynamic>>> fetchDocuments(String collection) async {
    return await sources[1].fetchDocuments(collection);
  }

  Future<Map<String, dynamic>> fetchDocument(String collection, String uid) async {
    Map<String, dynamic> item;
    var source;
    if (uid != null) {
      for (source in sources) {
        item = await source.fetchDocument(collection, uid);
        if (item != null) {
          break;
        }
      }
      for (var cache in caches) {
        if (cache != source) {
          cache.addItem(collection, uid, item);
        }
      }
    }
    return item;
  }

  addItem(String collection, String uid, Map<String, dynamic> item) {
    if (uid != null) {
      for (var cache in caches) {
        cache.addItem(collection, uid, item);
      }
    }
  }

  storeFile(String collection, String inspectionId, File file, String contentType) async {
    String result = '';
    for (var cache in caches) {
      String str = await cache.storeFile(collection, inspectionId, file, contentType);
      if (str.isNotEmpty) {
        result = str;
      }
    }
    return result;
  }

  clearCache(String collection) async {
    for (var cache in caches) {
      await cache.clear(collection);
    }
  }
}

abstract class RepositorySource {
  Future<List<Map<String, dynamic>>> fetchDocuments(String collection);
  Future<Map<String, dynamic>> fetchDocument(String collection, String uid);
}

abstract class RepositoryCache {
  Future<int> addItem(String collection, String uid, Map<String, dynamic> item);
  Future<int> clear(String collection);
  Future<String> storeFile(String collection, String inspectionId, File file, String contentType);
}
