import 'package:path_provider/path_provider.dart';
import 'dart:io';
import 'package:path/path.dart' as PathUtil;
import 'package:image/image.dart' as ImageUtil;
import 'package:image_picker/image_picker.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:kova_resources/src/constants.dart';

class Utils {
  Future<Directory> getLocalDirectory() {
    return getApplicationDocumentsDirectory().then((Directory directory) {
      return directory;
    }).catchError((onError) {
      return null;
    }).whenComplete(() {});
  }

  Future<Directory> getStorageDirectory() {
    return getLocalDirectory().then((Directory localDirectory) {
      String localPath = localDirectory.path;
      String storagePath = '$localPath/storage';
      Directory storageDirectory = Directory(storagePath);
      return storageDirectory.exists().then((bool directoryExists) {
        if (directoryExists) {
          return storageDirectory;
        } else {
          storageDirectory.create(recursive: false);
          return storageDirectory;
        }
      }).catchError((onError) {
        return false;
      }).whenComplete(() {});
    }).catchError((onError) {
      return null;
    }).whenComplete(() {});
  }

  Future<bool> doesFileExist(String fileName) {
    return getStorageDirectory().then((Directory storageDirectory) {
      String storagePath = storageDirectory.path;
      File file = File('$storagePath/$fileName');
      return file.exists().then((bool fileExists) {
        if (fileExists) {
          return true;
        } else {
          return false;
        }
      }).catchError((onError) {
        return null;
      }).whenComplete(() {});
    }).catchError((onError) {
      return null;
    }).whenComplete(() {});
  }

  String getFileName(File file) {
    return PathUtil.basename(file.path);
  }

  Future<File> getPNGFile(File file, String fileId) async {
    Directory storageDirectory = await getStorageDirectory();
    ImageUtil.Image image = ImageUtil.decodeImage(file.readAsBytesSync());
    String newFileName = '$fileId.png';
    String newPath = '${storageDirectory.path}/$newFileName';
    return File(newPath)..writeAsBytesSync(ImageUtil.encodePng(image));
  }

  Future<File> getThumbFromFile(File file) async {
    ImageUtil.Image image = ImageUtil.decodeImage(file.readAsBytesSync());
    ImageUtil.Image thumb = ImageUtil.copyResize(image, width: 120);
    String fileName = PathUtil.basename(file.path);
    String filePath = file.path;
    filePath = filePath.replaceAll(fileName, '');
    fileName = fileName.replaceAll('.png', '_120.png');
    return File('$filePath$fileName')..writeAsBytesSync(ImageUtil.encodePng(thumb));
  }

  Future<File> selectImage() async {
    final File selectedFile = await ImagePicker.pickImage(source: ImageSource.gallery, imageQuality: 100);
    if (selectedFile == null) return null;
    String filePath = selectedFile.path;
    String originalFileName = PathUtil.basename(selectedFile.path);
    filePath = filePath.replaceAll(originalFileName, '');
    String fileName = originalFileName.replaceAll('scaled_', '');
    File actualFile = File('$filePath$fileName');
    if (fileName != originalFileName) {
      selectedFile.delete();
    }
    return actualFile;
  }

  Future<bool> showToast(String message) async {
    bool toast = await Fluttertoast.showToast(
      msg: message,
      toastLength: Toast.LENGTH_SHORT,
      gravity: ToastGravity.CENTER,
      timeInSecForIos: 2,
      backgroundColor: Constants.kColorPrimary,
      textColor: Constants.kColorWhite,
      fontSize: 16.0,
    );
    return toast;
  }
}
